Index: doc/options.txt
===================================================================
--- doc/options.txt	(revision 4615)
+++ doc/options.txt	(working copy)
@@ -637,7 +637,7 @@
 problems. Default value is 0 (disabled) because it's not a
 completely reliable heuristic.
 
-;--check-routing-island-len=INTEGER
+;--report-routing-islands
 : 	Routing islands are small road networks which are not connected to other
 roads. A typical case is a footway that is not connected to the main road
 network, or a small set of ways on the inner courtyard of a large building.
@@ -644,30 +644,19 @@
 :	These islands can cause problems if you try to calculate a route and the GPS
 selects a point on the island as a start or end. It will fail to calculate the
 route even if a major road is only a few steps away. If this option is
-specified, then mkgmap will detect these islands. If the value is set to zero,
-mkgmap will simply report the islands (you will need to set
-uk.me.parabola.imgfmt.app.net.RoadNetwork.level=INFO to activate logging of
-the message). If the value is greater than zero, mkgmap will mark islands with
-a total length less than the specified value in metres as not routable.
-Reasonable values are 500 or higher. The default is for the check to not take
-place. If any of the roads forming the island touches a tile boundary or a
-country border the island is ignored, as it may be connected to other roads in
-a different tile.
-:	See also option --add-boundary-nodes-at-admin-boundaries.
-: This option seems to cause routing problems in BaseCamp.
+specified, then mkgmap will report these islands.
+:	See also --max-routing-island-len.
 
 ;--report-similar-arcs
 : 	Issue a warning when more than one arc connects two nodes and
 the ways that the arcs are derived from contain identical
-points. It doesn't make sense to use this option at the same
-time as using the cycleway creating options.
+points.
 
-;--report-dead-ends=LEVEL
-: 	Set the dead end road warning level. The value of LEVEL (which
-defaults to 1 if this option is not specified) determines
+;--report-dead-ends[=LEVEL]
+: 	Set the dead end road warning level. The value of LEVEL determines
 those roads to report:
-:* 0 = none
-:* 1 = report on connected one-way roads that go nowhere
+:* 0 = none (the default)
+:* 1 = report on connected one-way roads that go nowhere (default if no LEVEL specified)
 :* 2 = also report on individual one-way roads that go nowhere.
 
 ;--dead-ends[=key[=value]][,key[=value]...]
@@ -981,12 +970,34 @@
 The tag mkgmap:drawLevel can be used to override the
 natural area of a polygon, so forcing changes to the rendering order.
 
+;--max-routing-island-len=integer
+: 	Routing islands are small road networks which are not connected to other
+roads. A typical case is a footway that is not connected to the main road
+network, or a small set of ways on the inner courtyard of a large building.
+:	These islands can cause problems if you try to calculate a route and the GPS
+selects a point on the island as a start or end. It will fail to calculate the
+route even if a major road is only a few steps away. If a positive value is
+specified, then mkgmap will mark islands with a total length less than the
+specified value in metres as not routable.
+Reasonable values are 500 or higher. The default is to not to mark any islands
+as unroutable. If any of the roads forming the island touches a tile boundary or a
+country border the island is ignored, as it may be connected to other roads in
+a different tile.
+:	See also options --add-boundary-nodes-at-admin-boundaries and
+--report-routing-islands.
+: This option seems to cause routing problems in BaseCamp.
+
 === Deprecated and Obsolete Options ===
 
+;--check-routing-island-len=integer
+: 	Deprecated; use --report-routing-islands and --max-routing-island-len instead.
+Translated to --report-routing-islands if info level logging is enabled, plus
+--max-routing-island-len=integer.
+
 ;--drive-on-left
 ;--drive-on-right
-: 	Deprecated; use drive-on instead.
-The options are translated to drive-on=left|right.
+: 	Deprecated; use --drive-on instead.
+The options are translated to --drive-on=left|right.
 
 ;--make-all-cycleways
 :   Deprecated; use --make-opposite-cycleways instead. Former meaning:
Index: src/uk/me/parabola/imgfmt/app/FileBackedImgFileWriter.java
===================================================================
--- src/uk/me/parabola/imgfmt/app/FileBackedImgFileWriter.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/app/FileBackedImgFileWriter.java	(working copy)
@@ -24,6 +24,7 @@
 import uk.me.parabola.imgfmt.Sized;
 import uk.me.parabola.imgfmt.fs.ImgChannel;
 import uk.me.parabola.imgfmt.sys.FileLink;
+import uk.me.parabola.log.Logger;
 
 /**
  * Write img file data to a temporary file. On a call to sync() the data
@@ -70,7 +71,7 @@
 			channel.transferTo(0, channel.size(), outputChan);
 		} finally {
 			if (!tmpFile.delete())
-				System.err.println("could not delete temporary file " + tmpFile.getPath());
+				Logger.Write("could not delete temporary file " + tmpFile.getPath());
 		}
 	}
 
Index: src/uk/me/parabola/imgfmt/app/lbl/LBLFileReader.java
===================================================================
--- src/uk/me/parabola/imgfmt/app/lbl/LBLFileReader.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/app/lbl/LBLFileReader.java	(working copy)
@@ -29,6 +29,7 @@
 import uk.me.parabola.imgfmt.app.labelenc.DecodedText;
 import uk.me.parabola.imgfmt.app.trergn.Subdivision;
 import uk.me.parabola.imgfmt.fs.ImgChannel;
+import uk.me.parabola.log.Logger;
 
 import static uk.me.parabola.imgfmt.app.Label.NULL_LABEL;
 
@@ -444,7 +445,7 @@
 			}
 
 			if (hasTides) {
-				System.out.println("Map has tide prediction, please implement!");
+				Logger.Write("Map has tide prediction, please implement!");
 			}
 
 			pois.put(poiOffset, poi);
Index: src/uk/me/parabola/imgfmt/app/mdr/MDRFile.java
===================================================================
--- src/uk/me/parabola/imgfmt/app/mdr/MDRFile.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/app/mdr/MDRFile.java	(working copy)
@@ -28,6 +28,7 @@
 import uk.me.parabola.imgfmt.app.srt.Sort;
 import uk.me.parabola.imgfmt.app.trergn.Point;
 import uk.me.parabola.imgfmt.fs.ImgChannel;
+import uk.me.parabola.log.Logger;
 
 /**
  * The MDR file.  This is embedded into a .img file, either its own
@@ -150,7 +151,7 @@
 		Sort sort = mdrHeader.getSort();
 
 		if (sort.getCodepage() != codePage)
-			System.err.println("WARNING: input files have different code pages");
+			Logger.Write("WARNING: input files have different code pages");
 	}
 
 	public Mdr14Record addCountry(Country country) {
Index: src/uk/me/parabola/imgfmt/app/net/NumberStyle.java
===================================================================
--- src/uk/me/parabola/imgfmt/app/net/NumberStyle.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/app/net/NumberStyle.java	(working copy)
@@ -1,5 +1,7 @@
 package uk.me.parabola.imgfmt.app.net;
 
+import uk.me.parabola.log.Logger;
+
 /**
  * The number style down one side of a side of a road.
  *
@@ -63,7 +65,7 @@
 		case 'O': return ODD;
 		case 'B': return BOTH;
 		case '0':
-			System.err.println("zero instead of capital O in number spec");
+			Logger.Write("zero instead of capital O in number spec");
 			return ODD;
 		default: return NONE;
 		}
Index: src/uk/me/parabola/imgfmt/app/net/RoadNetwork.java
===================================================================
--- src/uk/me/parabola/imgfmt/app/net/RoadNetwork.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/app/net/RoadNetwork.java	(working copy)
@@ -61,8 +61,8 @@
 	private int maxFlareLengthRatio ;
 	private boolean reportSimilarArcs;
 	private boolean routable;
-
-	private long maxSumRoadLenghts;
+	private boolean reportRoutingIslands;
+	private long maxSumRoadLengths;
 	/** for route island search */
 	private int visitId;  
 
@@ -71,7 +71,14 @@
 		checkRoundaboutFlares = props.getProperty("check-roundabout-flares", false);
 		maxFlareLengthRatio = props.getProperty("max-flare-length-ratio", 0);
 		reportSimilarArcs = props.getProperty("report-similar-arcs", false);
-		maxSumRoadLenghts = props.getProperty("check-routing-island-len", -1);
+		int checkRoutingIslandLengths = props.getProperty("check-routing-island-len", -1);
+		if (checkRoutingIslandLengths >= 0) {
+			Logger.Write("The --check-routing-island-len option is deprecated. Please use --check-routing-islands and/or max-routing-island-len");
+			maxSumRoadLengths = checkRoutingIslandLengths;
+			reportRoutingIslands = log.isInfoEnabled();
+		}
+		maxSumRoadLengths = props.getProperty("max-routing-island-len", -1);
+		reportRoutingIslands = props.getProperty("report-routing-islands", false);
 		routable = props.containsKey("route");
 		angleChecker.config(props);
 	}
@@ -303,8 +310,9 @@
 	 * report routing islands and maybe remove them from NOD.  
 	 */
 	private void checkRoutingIslands() {
-		if (maxSumRoadLenghts < 0)
+		if (maxSumRoadLengths <= 0 && !reportRoutingIslands)
 			return; // island check is disabled
+		
 		long t1 = System.currentTimeMillis();
 
 		// calculate all islands
@@ -314,7 +322,7 @@
 		if (!islands.isEmpty()) {
 			analyseIslands(islands);
 		}
-		if (maxSumRoadLenghts > 0) {
+		if (maxSumRoadLengths > 0) {
 			long t3 = System.currentTimeMillis();
 			log.info("routing island removal took", (t3 - t2), "ms");
 		}
@@ -350,10 +358,11 @@
 		for (List<RouteNode> island : islands) {
 			// compute size of island as sum of road lengths
 			Set<RoadDef> visitedRoads = new HashSet<>();
-			long sumOfRoadLenghts = calcIslandSize(island, nodeToRoadMap, visitedRoads);
-			log.info("Routing island at", island.get(0).getCoord().toDegreeString(), "with", island.size(),
-					"routing node(s) and total length of", sumOfRoadLenghts, "m");
-			if (sumOfRoadLenghts < maxSumRoadLenghts) {
+			long sumOfRoadLengths = calcIslandSize(island, nodeToRoadMap, visitedRoads);
+			if (reportRoutingIslands)
+				log.write("Routing island " + visitedRoads.iterator().next() +  " at " + island.get(0).getCoord().toDegreeString() + " with " + island.size() +
+					" routing node(s) and total length of " + sumOfRoadLengths + "m");
+			if (sumOfRoadLengths < maxSumRoadLengths) {
 				// set discarded flag for all nodes of the island
 				island.forEach(RouteNode::discard);
 				visitedRoads.forEach(rd -> rd.skipAddToNOD(true));
Index: src/uk/me/parabola/imgfmt/app/net/RouteArc.java
===================================================================
--- src/uk/me/parabola/imgfmt/app/net/RouteArc.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/app/net/RouteArc.java	(working copy)
@@ -321,8 +321,6 @@
 			} else {
 				writer.put(directionFromDegrees(initialHeading));
 			}
-		} else {
-//			System.out.println("skipped writing of initial dir");
 		}
 		if (haveCurve) {
 			int[] curvedat = encodeCurve();
Index: src/uk/me/parabola/imgfmt/app/net/RouteNode.java
===================================================================
--- src/uk/me/parabola/imgfmt/app/net/RouteNode.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/app/net/RouteNode.java	(working copy)
@@ -388,7 +388,7 @@
 							// non roundabout highway overlaps roundabout
 							nonRoundaboutArcs.remove(ra1);
 							if(!ra.getRoadDef().messagePreviouslyIssued("roundabout forks/overlaps"))
-								log.warn("Highway",ra1.getRoadDef(), "overlaps roundabout", ra.getRoadDef(), "at",coord.toOSMURL());
+								log.write("Highway " + ra1.getRoadDef() + " overlaps roundabout " + ra.getRoadDef() + " at " + coord.toOSMURL());
 							break;
 						}						
 					}
@@ -461,27 +461,27 @@
 			
 				RouteArc roundaboutArc = roundaboutArcs.get(0);
 				if (arcs.size() > 1 && roundaboutArcs.size() == 1)
-					log.warn("Roundabout",roundaboutArc.getRoadDef(),roundaboutArc.isForward() ? "starts at" : "ends at", coord.toOSMURL());
+					log.write("Roundabout " + roundaboutArc.getRoadDef() + (roundaboutArc.isForward() ? " starts at " : " ends at ") + coord.toOSMURL());
 				if (countNonRoundaboutRoads > 1)
-					log.warn("Roundabout",roundaboutArc.getRoadDef(),"is connected to more than one road at",coord.toOSMURL());
+					log.write("Roundabout " + roundaboutArc.getRoadDef() + " is connected to more than one road at " + coord.toOSMURL());
 				else if (countNonRoundaboutRoads == 1) {
 					if (countNonRoundaboutOtherHighways > 0) {
 						if (countHighwaysInsideRoundabout > 0)
-							log.warn("Roundabout",roundaboutArc.getRoadDef(),"is connected to a road",countNonRoundaboutOtherHighways,"other highway(s) and",countHighwaysInsideRoundabout,"highways inside the roundabout at",coord.toOSMURL());
+							log.write("Roundabout " + roundaboutArc.getRoadDef() + " is connected to a road, " + countNonRoundaboutOtherHighways + " other highway(s) and " + countHighwaysInsideRoundabout + " highways inside the roundabout at " + coord.toOSMURL());
 						else
-							log.warn("Roundabout",roundaboutArc.getRoadDef(),"is connected to a road and",countNonRoundaboutOtherHighways,"other highway(s) at",coord.toOSMURL());
+							log.write("Roundabout " + roundaboutArc.getRoadDef() + " is connected to a road and " + countNonRoundaboutOtherHighways + " other highway(s) at " + coord.toOSMURL());
 					}
 					else if (countHighwaysInsideRoundabout > 0)
-						log.warn("Roundabout",roundaboutArc.getRoadDef(),"is connected to a road and",countHighwaysInsideRoundabout,"highway(s) inside the roundabout at",coord.toOSMURL());
+						log.write("Roundabout " + roundaboutArc.getRoadDef() + " is connected to a road and " + countHighwaysInsideRoundabout + " highway(s) inside the roundabout at " + coord.toOSMURL());
 				}
 				else if (countNonRoundaboutOtherHighways > 0) {
 					if (countHighwaysInsideRoundabout > 0)
-						log.warn("Roundabout",roundaboutArc.getRoadDef(),"is connected to",countNonRoundaboutOtherHighways,"highway(s) and",countHighwaysInsideRoundabout,"inside the roundabout at",coord.toOSMURL());
+						log.write("Roundabout " + roundaboutArc.getRoadDef() + " is connected to " + countNonRoundaboutOtherHighways+ " highway(s) and " + countHighwaysInsideRoundabout + " inside the roundabout at " + coord.toOSMURL());
 					else if (countNonRoundaboutOtherHighways > 1)
-						log.warn("Roundabout",roundaboutArc.getRoadDef(),"is connected to",countNonRoundaboutOtherHighways,"highways at",coord.toOSMURL());
+						log.write("Roundabout " + roundaboutArc.getRoadDef() + " is connected to " + countNonRoundaboutOtherHighways + " highways at " + coord.toOSMURL());
 				}
 				else if (countHighwaysInsideRoundabout > 1)
-					log.warn("Roundabout",roundaboutArc.getRoadDef(),"is connected to",countHighwaysInsideRoundabout,"highways inside the roundabout at",coord.toOSMURL());
+					log.write("Roundabout " +roundaboutArc.getRoadDef() + " is connected to " + countHighwaysInsideRoundabout + " highways inside the roundabout at " + coord.toOSMURL());
 				if(roundaboutArcs.size() > 2) {
 					for(RouteArc fa : roundaboutArcs) {
 						if(fa.isForward()) {
@@ -492,12 +492,12 @@
 									   ((fb.isForward() && fb.getDest() == fa.getDest()) ||
 										(!fb.isForward() && fb.getSource() == fa.getDest()))) {
 										if(!rd.messagePreviouslyIssued("roundabout forks/overlaps")) {
-											log.warn("Roundabout " + rd + " overlaps " + fb.getRoadDef() + " at " + coord.toOSMURL());
+											log.write("Roundabout " + rd + " overlaps " + fb.getRoadDef() + " at " + coord.toOSMURL());
 										}
 									}
 									else if (fb.isForward()
 											&& !rd.messagePreviouslyIssued("roundabout forks/overlaps")) {
-										log.warn("Roundabout " + rd + " forks at " + coord.toOSMURL());
+										log.write("Roundabout " + rd + " forks at " + coord.toOSMURL());
 									}
 								}
 							}
@@ -641,14 +641,14 @@
 
 						// only issue one warning per flare
 						if(!fa.isForward())
-							log.warn("Outgoing roundabout flare road " + fa.getRoadDef() + " points in wrong direction? " + fa.getSource().coord.toOSMURL());
+							log.write("Outgoing roundabout flare road " + fa.getRoadDef() + " points in wrong direction? " + fa.getSource().coord.toOSMURL());
 						else if(fb.isForward())
-							log.warn("Incoming roundabout flare road " + fb.getRoadDef() + " points in wrong direction? " + fb.getSource().coord.toOSMURL());
+							log.write("Incoming roundabout flare road " + fb.getRoadDef() + " points in wrong direction? " + fb.getSource().coord.toOSMURL());
 						else if(!fa.getRoadDef().isOneway())
-							log.warn("Outgoing roundabout flare road " + fa.getRoadDef() + " is not oneway? " + fa.getSource().coord.toOSMURL());
+							log.write("Outgoing roundabout flare road " + fa.getRoadDef() + " is not oneway? " + fa.getSource().coord.toOSMURL());
 
 						else if(!fb.getRoadDef().isOneway())
-							log.warn("Incoming roundabout flare road " + fb.getRoadDef() + " is not oneway? " + fb.getDest().coord.toOSMURL());
+							log.write("Incoming roundabout flare road " + fb.getRoadDef() + " is not oneway? " + fb.getDest().coord.toOSMURL());
 						else {
 							// check that the flare road arcs are not
 							// part of a longer way
@@ -655,9 +655,9 @@
 							for(RouteArc a : fa.getDest().arcs) {
 								if(a.isDirect() && a.getDest() != this && a.getDest() != nb) {
 									if(a.getRoadDef() == fa.getRoadDef())
-										log.warn("Outgoing roundabout flare road " + fb.getRoadDef() + " does not finish at flare? " + fa.getDest().coord.toOSMURL());
+										log.write("Outgoing roundabout flare road " + fb.getRoadDef() + " does not finish at flare? " + fa.getDest().coord.toOSMURL());
 									else if(a.getRoadDef() == fb.getRoadDef())
-										log.warn("Incoming roundabout flare road " + fb.getRoadDef() + " does not start at flare? " + fb.getDest().coord.toOSMURL());
+										log.write("Incoming roundabout flare road " + fb.getRoadDef() + " does not start at flare? " + fb.getDest().coord.toOSMURL());
 								}
 							}
 						}
@@ -670,16 +670,19 @@
 	public void reportSimilarArcs() {
 		for(int i = 0; i < arcs.size(); ++i) {
 			RouteArc arci = arcs.get(i);
-			if (!arci.isDirect())
+			RoadDef rdi = arci.getRoadDef();
+			if (!arci.isDirect() || rdi.isSynthesised())
 				continue;
 			for(int j = i + 1; j < arcs.size(); ++j) {
 				RouteArc arcj = arcs.get(j);
-				if (!arcj.isDirect())
+				RoadDef rdj = arcj.getRoadDef();
+				if (!arcj.isDirect() || rdj.isSynthesised())
 					continue;
 				if(arci.getDest() == arcj.getDest() &&
 				   arci.getLength() == arcj.getLength() &&
-				   arci.getPointsHash() == arcj.getPointsHash()) {
-					log.warn("Similar arcs (" + arci.getRoadDef() + " and " + arcj.getRoadDef() + ") from " + coord.toOSMURL());
+				   arci.getPointsHash() == arcj.getPointsHash() &&
+				   !rdi.messagePreviouslyIssued("Similar arcs")) {
+					log.write("Similar arcs " + rdi + " and " + rdj + " found at " + coord.toOSMURL());
 				}
 			}
 		}
Index: src/uk/me/parabola/imgfmt/app/srt/SrtFileReader.java
===================================================================
--- src/uk/me/parabola/imgfmt/app/srt/SrtFileReader.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/app/srt/SrtFileReader.java	(working copy)
@@ -32,6 +32,7 @@
 import uk.me.parabola.imgfmt.fs.ImgChannel;
 import uk.me.parabola.imgfmt.sys.FileImgChannel;
 import uk.me.parabola.imgfmt.sys.ImgFS;
+import uk.me.parabola.log.Logger;
 
 /**
  * The sort file.
@@ -281,7 +282,7 @@
 				chan.close();
 			}
 		} catch (FileNotFoundException e) {
-			System.err.println("Could not open file: " + infile);
+			Logger.Write("Could not open file: " + infile);
 		} finally {
 			if (fs != null) {
 				fs.close();
Index: src/uk/me/parabola/imgfmt/app/trergn/ExtTypeAttributes.java
===================================================================
--- src/uk/me/parabola/imgfmt/app/trergn/ExtTypeAttributes.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/app/trergn/ExtTypeAttributes.java	(working copy)
@@ -1115,8 +1115,6 @@
 				range = Double.parseDouble(parts[1]);
 			if(parts.length > 2)
 				angle = Double.parseDouble(parts[2]);
-
-			//System.err.println("light = " + this);
 		}
 
 		public String toString() {
Index: src/uk/me/parabola/imgfmt/app/typ/TypElement.java
===================================================================
--- src/uk/me/parabola/imgfmt/app/typ/TypElement.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/app/typ/TypElement.java	(working copy)
@@ -136,7 +136,6 @@
 			} catch (CharacterCodingException ignore) {
 				//ignore.printStackTrace();
 				String name = encoder.charset().name();
-				//System.out.println("cs " + name);
 				log.warn("Cannot represent String", tl.getText(), "for language", tl.getLang(), "in CodePage", name);
 				//throw new TypLabelException(name);
 			}
Index: src/uk/me/parabola/imgfmt/mdxfmt/MdxFile.java
===================================================================
--- src/uk/me/parabola/imgfmt/mdxfmt/MdxFile.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/mdxfmt/MdxFile.java	(working copy)
@@ -23,6 +23,8 @@
 import java.util.ArrayList;
 import java.util.List;
 
+import uk.me.parabola.log.Logger;
+
 /**
  * The MDX index file.  Used with the global index.  This is located
  * at the family level in the windows registry and can perhaps index
@@ -103,7 +105,7 @@
 			// Although its not necessarily wrong for them to be zero, it probably
 			// sign that something is wrong.
 			if (info.getHexMapname() == 0 || info.getMapname() == 0)
-				System.err.println("Invalid mapname for " + info.getFilename() + ", perhaps it is not a .img file");
+				Logger.Write("Invalid mapname for " + info.getFilename() + ", perhaps it is not a .img file");
 
 			buf.compact();
 			info.write(buf);
Index: src/uk/me/parabola/imgfmt/sys/ImgHeader.java
===================================================================
--- src/uk/me/parabola/imgfmt/sys/ImgHeader.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/sys/ImgHeader.java	(working copy)
@@ -188,7 +188,6 @@
 	 */
 	private void writeSizeValues(int blockSize) {
 		int endSector = (int) (((numBlocks+1L) * blockSize + 511) / 512);
-		//System.out.printf("end sector %d %x\n", endSector, endSector);
 
 		// We have three maximum values for sectors, heads and cylinders.  We attempt to find values
 		// for them that are larger than the 
@@ -205,7 +204,6 @@
 			for (int s : asList(4, 8, 16, 32)) {
 				for (int c : asList(0x20, 0x40, 0x80, 0x100, 0x200, 0x3ff)) {
 					log.info("shc=", s + "," + h + "," + c, "end=", endSector);
-					//System.out.println("shc=" + s + "," + h + "," + c + "end=" + endSector);
 					if (s * h * c > endSector) {
 						headsPerCylinder = h;
 						sectorsPerTrack = s;
Index: src/uk/me/parabola/imgfmt/Utils.java
===================================================================
--- src/uk/me/parabola/imgfmt/Utils.java	(revision 4615)
+++ src/uk/me/parabola/imgfmt/Utils.java	(working copy)
@@ -31,6 +31,7 @@
 
 import uk.me.parabola.imgfmt.app.Coord;
 import uk.me.parabola.imgfmt.app.ImgFileWriter;
+import uk.me.parabola.log.Logger;
 /**
  * Some miscellaneous functions that are used within the .img code.
  *
@@ -189,7 +190,7 @@
 			try {
 				f.close();
 			} catch (IOException e) {
-				e.printStackTrace();
+				Logger.WriteStackTrace(e);
 			}
 		}
 	}
Index: src/uk/me/parabola/log/Logger.java
===================================================================
--- src/uk/me/parabola/log/Logger.java	(revision 4615)
+++ src/uk/me/parabola/log/Logger.java	(working copy)
@@ -20,6 +20,8 @@
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.PrintWriter;
+import java.io.StringWriter;
 import java.util.Properties;
 import java.util.logging.ConsoleHandler;
 import java.util.logging.Level;
@@ -35,6 +37,7 @@
  */
 public class Logger {
 	private final java.util.logging.Logger log;
+	private static final java.util.logging.Logger DefaultLogger = java.util.logging.Logger.getGlobal();
 
 	private static final ThreadLocal<String> threadTags = new ThreadLocal<>();
 
@@ -195,8 +198,9 @@
 	}
 
 	public void error(Object ... olist) {
-			arrayFormat(Level.SEVERE, olist);
+		arrayFormat(Level.SEVERE, olist);
 	}
+
 	public void errorf(String fmt, Object... args) {
 		printf(Level.SEVERE, fmt, args);
 	}
@@ -205,6 +209,14 @@
 		log.log(Level.SEVERE, tagMessage(o == null? "null" : o.toString()), e);
 	}
 
+	public void write(Object o) {
+		log.log(Level.OFF, tagMessage(o == null? "null" : o.toString()));
+	}
+
+	public void write(Object ... olist) {
+		arrayFormat(Level.OFF, olist);
+	}
+	
 	public void log(Level level, Object o) {
 		if (log.isLoggable(level))
 			log.log(level, tagMessage(o == null? "null" : o.toString()));
@@ -215,6 +227,27 @@
 			arrayFormat(level, olist);
 	}
 	
+	// forced write, overriding log level and omitting any prefixes
+	public static void Write(Object o) {
+		if (o != null)
+			DefaultLogger.log(Level.OFF, o.toString());
+	}
+	
+	public static void WriteStackTrace(Throwable t)
+	{
+		StringWriter out = new StringWriter();
+		PrintWriter pw = new PrintWriter(out);
+		t.printStackTrace(pw);
+		DefaultLogger.log(Level.SEVERE, out.toString());
+	}
+
+	public static void WriteStackTrace(String msg, Throwable t)
+	{
+		StringWriter out = new StringWriter();
+		PrintWriter pw = new PrintWriter(out);
+		t.printStackTrace(pw);
+		DefaultLogger.log(Level.SEVERE, msg + " " + out.toString());
+	}
 	/**
 	 * Format the list of arguments by appending them to one string, keeping a
 	 * space between them.
Index: src/uk/me/parabola/log/UsefulFormatter.java
===================================================================
--- src/uk/me/parabola/log/UsefulFormatter.java	(revision 4615)
+++ src/uk/me/parabola/log/UsefulFormatter.java	(working copy)
@@ -20,6 +20,7 @@
 import java.io.StringWriter;
 import java.util.Calendar;
 import java.util.logging.Formatter;
+import java.util.logging.Level;
 import java.util.logging.LogRecord;
 
 /**
@@ -38,29 +39,30 @@
 	public String format(LogRecord record) {
 		StringBuffer sb = new StringBuffer();
 
-		if (showTime) {
-			long millis = record.getMillis();
-			Calendar cal = Calendar.getInstance();
-			cal.setTimeInMillis(millis);
-			sb.append(cal.get(Calendar.YEAR));
-			sb.append('/');
-			sb.append(fmt2(cal.get(Calendar.MONTH)+1));
-			sb.append('/');
-			sb.append(fmt2(cal.get(Calendar.DAY_OF_MONTH)));
-			sb.append(' ');
-			sb.append(fmt2(cal.get(Calendar.HOUR_OF_DAY)));
-			sb.append(':');
-			sb.append(fmt2(cal.get(Calendar.MINUTE)));
-			sb.append(':');
-			sb.append(fmt2(cal.get(Calendar.SECOND)));
-			sb.append(' ');
+		if (record.getLevel() != Level.OFF) {
+			if (showTime) {
+				long millis = record.getMillis();
+				Calendar cal = Calendar.getInstance();
+				cal.setTimeInMillis(millis);
+				sb.append(cal.get(Calendar.YEAR));
+				sb.append('/');
+				sb.append(fmt2(cal.get(Calendar.MONTH)+1));
+				sb.append('/');
+				sb.append(fmt2(cal.get(Calendar.DAY_OF_MONTH)));
+				sb.append(' ');
+				sb.append(fmt2(cal.get(Calendar.HOUR_OF_DAY)));
+				sb.append(':');
+				sb.append(fmt2(cal.get(Calendar.MINUTE)));
+				sb.append(':');
+				sb.append(fmt2(cal.get(Calendar.SECOND)));
+				sb.append(' ');
+			}
+			
+			sb.append(record.getLevel().getLocalizedName());
+			sb.append(" (");
+			sb.append(shortName(record.getLoggerName()));
+			sb.append("): ");
 		}
-		
-		sb.append(record.getLevel().getLocalizedName());
-		sb.append(" (");
-		sb.append(shortName(record.getLoggerName()));
-		sb.append("): ");
-
 		sb.append(record.getMessage());
 		
 		sb.append(lineSeparator);
Index: src/uk/me/parabola/mkgmap/build/Locator.java
===================================================================
--- src/uk/me/parabola/mkgmap/build/Locator.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/build/Locator.java	(working copy)
@@ -187,8 +187,6 @@
 		{	
 			String[] cityList = p.getIsIn().split(",");
 			
-			//System.out.println(p.getIsIn());
-
 			// is_in content is not well defined so we try our best to get some info out of it
 			// Format 1 popular in Germany: "County,State,Country,Continent"
 
Index: src/uk/me/parabola/mkgmap/build/LocatorConfig.java
===================================================================
--- src/uk/me/parabola/mkgmap/build/LocatorConfig.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/build/LocatorConfig.java	(working copy)
@@ -180,13 +180,12 @@
 			}
 			else
 			{
-				System.out.println(fileName + "contains invalid root tag " + rootNode.getNodeName());
+				log.error(fileName + "contains invalid root tag " + rootNode.getNodeName());
 			}
-   	}
+		}
 		catch (Exception ex)
 		{
-			ex.printStackTrace();
-			//System.out.println("Something is wrong here");
+			Logger.WriteStackTrace(ex);
 		}
   	}
 
Index: src/uk/me/parabola/mkgmap/combiners/FileInfo.java
===================================================================
--- src/uk/me/parabola/mkgmap/combiners/FileInfo.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/combiners/FileInfo.java	(working copy)
@@ -204,7 +204,7 @@
 			fr.position(0x15);
 			info.setCodePage(fr.get2u());
 		} catch (IOException e) {
-			e.printStackTrace();
+			Logger.WriteStackTrace(e);
 		}
 	}
 
Index: src/uk/me/parabola/mkgmap/combiners/GmapiBuilder.java
===================================================================
--- src/uk/me/parabola/mkgmap/combiners/GmapiBuilder.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/combiners/GmapiBuilder.java	(working copy)
@@ -34,6 +34,7 @@
 import uk.me.parabola.imgfmt.fs.FileSystem;
 import uk.me.parabola.imgfmt.fs.ImgChannel;
 import uk.me.parabola.imgfmt.sys.ImgFS;
+import uk.me.parabola.log.Logger;
 import uk.me.parabola.mkgmap.CommandArgs;
 
 import static java.nio.file.StandardOpenOption.*;
@@ -99,7 +100,7 @@
 				typFile = info.getFilename();
 
 		} catch (IOException e) {
-			e.printStackTrace();
+			Logger.WriteStackTrace(e);
 		}
 	}
 
@@ -131,7 +132,7 @@
 			writeXmlFile(gmapDir);
 
 		} catch (IOException e) {
-			e.printStackTrace();
+			Logger.WriteStackTrace(e);
 		}
 	}
 
Index: src/uk/me/parabola/mkgmap/combiners/GmapsuppBuilder.java
===================================================================
--- src/uk/me/parabola/mkgmap/combiners/GmapsuppBuilder.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/combiners/GmapsuppBuilder.java	(working copy)
@@ -121,7 +121,7 @@
 			ImgChannel chan = imgFs.create(imgname);
 			mdrBuilder.initForDevice(chan, sort, mdrConfig);
 		} catch (FileExistsException e) {
-			System.err.println("Could not create duplicate MDR file");
+			log.error("Could not create duplicate MDR file", e);
 		}
 
 		mdrBuilderMap.put(familyId, mdrBuilder);
@@ -140,11 +140,9 @@
 			}
 		} else {
 			if (prevSort.getCodepage() != sort.getCodepage())
-				System.err.printf("WARNING: input file '%s' has a different code page (%d rather than %d)\n",
-						info.getFilename(), sort.getCodepage(), prevSort.getCodepage());
+				log.error("Input file " + info.getFilename() + " has a different code page (" + sort.getCodepage() + " rather than " + prevSort.getCodepage() + ")");
 			if (info.hasSortOrder() && prevSort.getSortOrderId() != sort.getSortOrderId())
-				System.err.printf("WARNING: input file '%s' has a different sort order (%x rather than %x\n",
-						info.getFilename(), sort.getSortOrderId(), prevSort.getSortOrderId());
+				log.error("Input file " + info.getFilename() + " has a different sort order (" + sort.getSortOrderId() + " rather than " + prevSort.getSortOrderId() + ")");
 		}
 	}
 
@@ -186,8 +184,7 @@
 			writeMpsFile();
 
 		} catch (FileNotWritableException e) {
-			log.warn("Could not create gmapsupp file");
-			System.err.println("Could not create gmapsupp file");
+			log.error("Could not create gmapsupp file", e);
 		} finally {
 			Utils.closeFile(imgFs);
 		}
Index: src/uk/me/parabola/mkgmap/combiners/NsisBuilder.java
===================================================================
--- src/uk/me/parabola/mkgmap/combiners/NsisBuilder.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/combiners/NsisBuilder.java	(working copy)
@@ -27,11 +27,13 @@
 import java.util.Map;
 
 import uk.me.parabola.imgfmt.Utils;
+import uk.me.parabola.log.Logger;
 import uk.me.parabola.mkgmap.CommandArgs;
 import uk.me.parabola.mkgmap.Version;
 
 
 public class NsisBuilder implements Combiner {
+	private Logger log = Logger.getLogger(NsisBuilder.class);
 	private String baseFilename;
 	private String nsisFilename;
 	private String licenseFilename;
@@ -95,7 +97,7 @@
 		} catch (Exception ex) {
 			inStream = this.getClass().getResourceAsStream("/installer/installer_template.nsi");
 			if (inStream == null) {
-				System.err.println("Could not find the installer template.");
+				log.error("Could not find the installer template.");
 				return;
 			}
 		}
@@ -118,7 +120,7 @@
 			}
 
 		} catch (IOException e) {
-			System.err.println("Could not write NSIS file");
+			log.error("Could not write NSIS file", e);
 		} finally {
 			Utils.closeFile(inStream);
 		}
@@ -182,7 +184,7 @@
 		} catch (Exception ex) {
 			inStream = this.getClass().getResourceAsStream("/installer/license_template.txt");
 			if (inStream == null) {
-				System.err.println("Could not find the license template.");
+				log.error("Could not find the license template.");
 				return;
 			}
 		}
@@ -198,7 +200,7 @@
 			}
 
 		} catch (IOException e) {
-			System.err.println("Could not write license file");
+			log.error("Could not write license file", e);
 		} finally {
 			Utils.closeFile(inStream);
 		}
Index: src/uk/me/parabola/mkgmap/combiners/OverviewBuilder.java
===================================================================
--- src/uk/me/parabola/mkgmap/combiners/OverviewBuilder.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/combiners/OverviewBuilder.java	(working copy)
@@ -215,7 +215,7 @@
 			codepage = finfo.getCodePage();
 		} 
 		if (codepage != finfo.getCodePage()){
-			System.err.println("WARNING: input file " + filename + " has different code page " + finfo.getCodePage());
+			log.write("WARNING: input file " + filename + " has different code page " + finfo.getCodePage());
 		}
 
 		try {
@@ -225,7 +225,7 @@
 				encodingType = mapReader.getEncodingType();
 			} 
 			if (encodingType != mapReader.getEncodingType()){
-				System.err.println("WARNING: input file " + filename + " has different charset type " + encodingType);
+				log.write("WARNING: input file " + filename + " has different charset type " + encodingType);
 			}
 
 			String[] msgs = mapReader.getCopyrights();
@@ -324,7 +324,6 @@
 			int min = levels[l].getLevel();
 			int res = levels[l].getResolution();
 			List<Polyline> lineList = mapReader.linesForLevel(min);
-			//System.out.println(lineList.size() + " lines in lowest resolution " + levels[1].getResolution());
 			for (Polyline line : lineList) {
 				if (log.isDebugEnabled())
 					log.debug("got line", line);
Index: src/uk/me/parabola/mkgmap/CommandArgs.java
===================================================================
--- src/uk/me/parabola/mkgmap/CommandArgs.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/CommandArgs.java	(working copy)
@@ -11,6 +11,7 @@
 
 import uk.me.parabola.imgfmt.ExitException;
 import uk.me.parabola.imgfmt.app.srt.Sort;
+import uk.me.parabola.log.Logger;
 import uk.me.parabola.util.EnhancedProperties;
 
 public class CommandArgs {
@@ -97,14 +98,14 @@
 		// Test if directory exists
 		File outputDir = new File(fileOutputDir);
 		if (!outputDir.exists()) {
-			System.out.println("Output directory not found. Creating directory '" + fileOutputDir + "'");
+			Logger.Write("Output directory not found. Creating directory '" + fileOutputDir + "'");
 			outputDir.mkdirs();
 			if (!outputDir.exists()) {
-				System.err.println("Unable to create output directory! Using default directory instead");
+				Logger.Write("Unable to create output directory! Using default directory instead");
 				fileOutputDir = DEFAULT_DIR;
 			}
 		} else if (!outputDir.isDirectory()) {
-			System.err.println("The --output-dir parameter must specify a directory. The parameter is being ignored, writing to default directory instead.");
+			Logger.Write("The --output-dir parameter must specify a directory. The parameter is being ignored, writing to default directory instead.");
 			fileOutputDir = DEFAULT_DIR;
 		}
 		
Index: src/uk/me/parabola/mkgmap/CommandArgsReader.java
===================================================================
--- src/uk/me/parabola/mkgmap/CommandArgsReader.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/CommandArgsReader.java	(working copy)
@@ -47,6 +47,7 @@
 	private final ArgumentProcessor proc;
 
 	private boolean mapnameWasSet;
+	private boolean hasFiles = false;
 
 	private final ArgList arglist = new ArgList();
 
@@ -109,6 +110,7 @@
 
 			} else {
 				log.debug("adding filename:", arg);
+				hasFiles = true;
 				add(new Filename(arg));
 			}
 		}
@@ -125,6 +127,9 @@
 			proc.endOptions(new CommandArgs(this.args));
 	}
 
+	public boolean getHasFiles() {
+		return hasFiles;
+	}
 
 	/**
 	 * Add an option based on the option and value separately.
@@ -202,6 +207,7 @@
 		case "input-file":
 			if (value != null){
 				log.debug("adding filename", value);
+				hasFiles = true;
 				add(new Filename(value));
 			}
 			break;
Index: src/uk/me/parabola/mkgmap/main/Main.java
===================================================================
--- src/uk/me/parabola/mkgmap/main/Main.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/main/Main.java	(working copy)
@@ -81,6 +81,7 @@
  */
 public class Main implements ArgumentProcessor {
 	private static final Logger log = Logger.getLogger(Main.class);
+	private static final Date StartTime = new Date();
 
 	// Final .img file combiners.
 	private final List<Combiner> combiners = new ArrayList<>();
@@ -126,7 +127,7 @@
 	 */
 	private static int mainStart(String... args) {
 		Instant start = Instant.now();
-		System.out.println("Time started: " + new Date());
+
 		// We need at least one argument.
 		if (args.length < 1) {
 			printUsage();
@@ -137,18 +138,18 @@
 		Main mm = new Main();
 
 		int numExitExceptions = 0;
+		CommandArgsReader commandArgs = new CommandArgsReader(mm);
 		try {
 			// Read the command line arguments and process each filename found.
-			CommandArgsReader commandArgs = new CommandArgsReader(mm);
 			commandArgs.setValidOptions(getValidOptions(System.err));
 			commandArgs.readArgs(args);
 		} catch (OutOfMemoryError e) {
 			++numExitExceptions;
-			System.err.println(e);
+			Logger.Write("Out of memory.");
 			if (mm.maxJobs > 1)
-				System.err.println("Try using the mkgmap --max-jobs option with a value less than " + mm.maxJobs  + " to reduce the memory requirement, or use the Java -Xmx option to increase the available heap memory.");
+				Logger.Write("Try using the mkgmap --max-jobs option with a value less than " + mm.maxJobs  + " to reduce the memory requirement, or use the Java -Xmx option to increase the available heap memory.");
 			else
-				System.err.println("Try using the Java -Xmx option to increase the available heap memory.");
+				Logger.Write("Try using the Java -Xmx option to increase the available heap memory.");
 		} catch (MapFailedException | ExitException e) {
 			// one of the combiners failed
 			++numExitExceptions;
@@ -158,30 +159,29 @@
 				message += "\r\n" + cause.toString();
 				cause = cause.getCause();
 			}
-			System.err.println(message);
+			Logger.Write(message);
 		}
 		
-		System.out.println("Number of ExitExceptions: " + numExitExceptions);
-		
-		System.out.println("Time finished: " + new Date());
-		Duration duration = Duration.between(start, Instant.now());
-		long seconds = duration.getSeconds();
-		if (seconds > 0) {
-			long hours = seconds / 3600;
-			seconds -= hours * 3600;
-			long minutes = seconds / 60;
-			seconds -= minutes * 60;
-			System.out.println("Total time taken: " + 
-								(hours > 0 ? hours + (hours > 1 ? " hours " : " hour ") : "") +
-								(minutes > 0 ? minutes + (minutes > 1 ? " minutes " : " minute ") : "") +
-								(seconds > 0 ? seconds + (seconds > 1 ? " seconds" : " second") : ""));
+		if(commandArgs.getHasFiles()) {
+			Logger.Write("Number of ExitExceptions: " + numExitExceptions);
+			
+			Logger.Write("Time finished: " + new Date());
+			Duration duration = Duration.between(start, Instant.now());
+			long seconds = duration.getSeconds();
+			if (seconds > 0) {
+				long hours = seconds / 3600;
+				seconds -= hours * 3600;
+				long minutes = seconds / 60;
+				seconds -= minutes * 60;
+				Logger.Write("Total time taken: " + 
+							 (hours > 0 ? hours + (hours > 1 ? " hours " : " hour ") : "") +
+							 (minutes > 0 ? minutes + (minutes > 1 ? " minutes " : " minute ") : "") +
+							 (seconds > 0 ? seconds + (seconds > 1 ? " seconds" : " second") : ""));
+			}
+			else
+				Logger.Write("Total time taken: " + duration.getNano() / 1000000 + " ms");
 		}
-		else
-			System.out.println("Total time taken: " + duration.getNano() / 1000000 + " ms");
-		if (numExitExceptions > 0 || mm.getProgramRC() != 0){
-			return 1;
-		}
-		return 0;
+		return (numExitExceptions > 0 || mm.getProgramRC() != 0) ? 1 : 0;
 	}
 	
 	private static void printUsage (){
@@ -342,15 +342,15 @@
 			else {
 				maxJobs = Integer.parseInt(val);
 				if (maxJobs < 1) {
-					log.warn("max-jobs has to be at least 1");
+					Logger.Write("max-jobs has to be at least 1");
 					maxJobs = 1;
 				}
 				if (maxJobs > Runtime.getRuntime().availableProcessors())
-					log.warn("It is recommended that max-jobs be no greater that the number of processor cores");
+					Logger.Write("It is recommended that max-jobs be no greater that the number of processor cores");
 			}
 			break;
 		case "version":
-			System.err.println(Version.VERSION);
+			System.out.println(Version.VERSION);
 			System.exit(0);
 		}
 	}
@@ -458,7 +458,7 @@
 		try {
 			style = new StyleImpl(styleFile, name, new EnhancedProperties(), performChecks);
 		} catch (SyntaxException e) {
-			System.err.println("Error in style: " + e.getMessage());
+			Logger.Write("Error in style: " + e.getMessage());
 		} catch (FileNotFoundException e) {
 			log.debug("could not find style", name);
 			try {
@@ -465,7 +465,7 @@
 				searchedStyleName = new File(styleFile).getName();
 				style = new StyleImpl(styleFile, null, new EnhancedProperties(), performChecks);
 			} catch (SyntaxException e1) {
-				System.err.println("Error in style: " + e1.getMessage());
+				Logger.Write("Error in style: " + e1.getMessage());
 			} catch (FileNotFoundException e1) {
 				log.debug("could not find style", styleFile);
 			}
@@ -485,14 +485,17 @@
 	public void endOptions(CommandArgs args) {
 		fileOptions(args);
 
+		int taskCount = futures.size();
+		if (taskCount > 0)
+			Logger.Write("Time started: " + StartTime);
+
 		log.info("Start tile processors");
 		int threadCount = maxJobs;
-		int taskCount = futures.size();
 		Runtime runtime = Runtime.getRuntime();
 		if (threadPool == null) {
 			if (threadCount == 0) {
 				threadCount = 1;
-				if (taskCount > 2) {
+				if ((taskCount > 2) && (runtime.availableProcessors() > 1)) {
 					//run one task to see how much memory it uses
 					log.info("Max Memory: " + runtime.maxMemory());
 					futures.get(0).run();
@@ -511,7 +514,7 @@
 					}
 					threadCount = Math.max(threadCount, 1);
 					threadCount = Math.min(threadCount, runtime.availableProcessors());
-					System.out.println("Setting max-jobs to " + threadCount);
+					Logger.Write("Setting max-jobs to " + threadCount);
 				}
 			}
 
@@ -566,14 +569,14 @@
 						throw new ExitException("Exiting - if you want to carry on regardless, use the --keep-going option");
 					}
 				} catch (Exception e) {
-					e.printStackTrace();
+					Logger.WriteStackTrace(e);
 					throw new ExitException("Exiting due to unexpected error");
 				}
 			}
 		}
-		System.out.println("Number of MapFailedExceptions: " + numMapFailedExceptions);
+		Logger.Write("Number of MapFailedExceptions: " + numMapFailedExceptions);
 		if ((taskCount > threadCount + 1) && (maxJobs == 0) && (threadCount < runtime.availableProcessors())) {
-			System.out.println("To reduce the run time, consider increasing the amnount of memory available for use by mkgmap by using the Java -Xmx flag to set the memory to more than " + 100* (1 + ((runtime.maxMemory() * runtime.availableProcessors()) / (threadCount * 1024 * 1024 * 100))) + " MB, providing this is less than the amount of physical memory installed.");
+			Logger.Write("To reduce the run time, consider increasing the amnount of memory available for use by mkgmap by using the Java -Xmx flag to set the memory to more than " + 100* (1 + ((runtime.maxMemory() * runtime.availableProcessors()) / (threadCount * 1024 * 1024 * 100))) + " MB, providing this is less than the amount of physical memory installed.");
 		}
 
 		if (combiners.isEmpty())
@@ -589,7 +592,7 @@
 			hasFiles = true;
 		}
 		if (!hasFiles){
-			log.warn("nothing to do for combiners.");
+			log.info("nothing to do for combiners.");
 			return;
 		}
 		log.info("Combining maps");
@@ -679,7 +682,7 @@
 				if (f.exists() && f.isFile()) {
 					try {
 						Files.delete(f.toPath());
-						log.warn("removed " + f);
+						log.info("removed " + f);
 					} catch (IOException e) {
 						log.warn("removing " + f + "failed with " + e.getMessage());
 					}
Index: src/uk/me/parabola/mkgmap/main/MapMaker.java
===================================================================
--- src/uk/me/parabola/mkgmap/main/MapMaker.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/main/MapMaker.java	(working copy)
@@ -51,7 +51,7 @@
 
 	public String makeMap(CommandArgs args, String filename) {
 		if (new File(filename).isDirectory()) {
-			System.err.println("Need a single file, not a directory: " + filename);
+			log.error("Need a single file, not a directory: " + filename);
 			return filename;
 		}
 		try {
@@ -69,11 +69,10 @@
 			}
 			return makeMap(args, src, "");
 		} catch (FormatException e) {
-			System.err.println("Bad file format: " + filename);
-			System.err.println(e.getMessage());
+			log.error("Bad file format: " + filename, e);
 			return filename;
 		} catch (FileNotFoundException e) {
-			System.err.println("Could not open file: " + filename);
+			log.error("Could not open file: " + filename, e);
 			return filename;
 		}
 	}
Index: src/uk/me/parabola/mkgmap/osmstyle/actions/EchoAction.java
===================================================================
--- src/uk/me/parabola/mkgmap/osmstyle/actions/EchoAction.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/osmstyle/actions/EchoAction.java	(working copy)
@@ -16,9 +16,10 @@
 package uk.me.parabola.mkgmap.osmstyle.actions;
 
 import uk.me.parabola.mkgmap.reader.osm.Element;
+import uk.me.parabola.log.Logger;
 
 /**
- * Sends a message to the console.
+ * Logs a message.
  * 
  * @author Richard Fairhurst
  */
@@ -30,7 +31,7 @@
 	}
 
 	public boolean perform(Element el) {
-		System.err.println(el.getBasicLogInformation() + " " + value.build(el, el));
+		Logger.Write(el.getBasicLogInformation() + " " + value.build(el, el));
 		return false;
 	}
 }
Index: src/uk/me/parabola/mkgmap/osmstyle/actions/EchoTagsAction.java
===================================================================
--- src/uk/me/parabola/mkgmap/osmstyle/actions/EchoTagsAction.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/osmstyle/actions/EchoTagsAction.java	(working copy)
@@ -14,9 +14,10 @@
 package uk.me.parabola.mkgmap.osmstyle.actions;
 
 import uk.me.parabola.mkgmap.reader.osm.Element;
+import uk.me.parabola.log.Logger;
 
 /**
- * Sends a message including the tags of an element to System.err.
+ * Logs a message including the tags of an element.
  * 
  * @author WanMil
  */
@@ -28,7 +29,7 @@
 	}
 
 	public boolean perform(Element el) {
-		System.err.println(el.getBasicLogInformation() + " " + el.toTagString() + " " + value.build(el, el));
+		Logger.Write(el.getBasicLogInformation() + " " + el.toTagString() + " " + value.build(el, el));
 		return false;
 	}
 	
Index: src/uk/me/parabola/mkgmap/osmstyle/eval/AbstractOp.java
===================================================================
--- src/uk/me/parabola/mkgmap/osmstyle/eval/AbstractOp.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/osmstyle/eval/AbstractOp.java	(working copy)
@@ -20,6 +20,7 @@
 import java.util.Set;
 
 import uk.me.parabola.imgfmt.ExitException;
+import uk.me.parabola.log.Logger;
 import uk.me.parabola.mkgmap.osmstyle.function.GetTagFunction;
 import uk.me.parabola.mkgmap.reader.osm.Element;
 import uk.me.parabola.mkgmap.scan.SyntaxException;
@@ -210,7 +211,7 @@
 		} else if (this.isType(NodeType.EXISTS) || this.isType(NodeType.NOT_EXISTS) || this.isType(NodeType.NOT)) {
 			set.addAll(getFirst().getEvaluatedTagKeys());
 		} else if (this.getFirst() != null) {
-			System.err.println("Unhandled type of Op");
+			Logger.Write("Unhandled type of Op");
 		}
 			
 	}
Index: src/uk/me/parabola/mkgmap/osmstyle/housenumber/HousenumberGenerator.java
===================================================================
--- src/uk/me/parabola/mkgmap/osmstyle/housenumber/HousenumberGenerator.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/osmstyle/housenumber/HousenumberGenerator.java	(working copy)
@@ -106,7 +106,7 @@
 		if (n != nameSearchDepth) {
 			nameSearchDepth = Math.min(25, Math.max(0, n));
 			if (nameSearchDepth != n) {
-				System.err.println("name-service-roads=" + n + " was changed to name-service-roads=" + nameSearchDepth);
+				Logger.Write("name-service-roads=" + n + " was changed to name-service-roads=" + nameSearchDepth);
 			}
 		}
 	}
Index: src/uk/me/parabola/mkgmap/osmstyle/StyledConverter.java
===================================================================
--- src/uk/me/parabola/mkgmap/osmstyle/StyledConverter.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/osmstyle/StyledConverter.java	(working copy)
@@ -204,7 +204,7 @@
 			countryAbbr = countryAbbr.toUpperCase();
 			
 		checkRoundabouts = props.getProperty("check-roundabouts",false);
-		reportDeadEnds = props.getProperty("report-dead-ends", 1);  
+		reportDeadEnds = (props.getProperty("report-dead-ends") != null) ? props.getProperty("report-dead-ends", 1) : 0;  
 		prefixSuffixFilter = new PrefixSuffixFilter(props);
 
 		lineAdder = line -> {
@@ -250,7 +250,7 @@
 				String optionKey = pair[0];
 				String tagKey = STYLE_OPTION_PREF + optionKey;
 				if (!style.getUsedTags().contains(tagKey)) {
-					System.err.println("Warning: Option style-options sets tag not used in style: '" 
+					Logger.Write("Warning: Option style-options sets tag not used in style: '" 
 							+ optionKey + "' (gives " + tagKey + ")");
 				} else {
 					String val = (pair.length == 1) ? "true" : pair[1];
@@ -264,7 +264,7 @@
 		if (style.getUsedTags() != null) {
 			for (String s : style.getUsedTags()) {
 				if (s != null && s.startsWith(STYLE_OPTION_PREF) && styleTags.get(s) == null) {
-					System.err.println("Warning: Option style-options doesn't specify '"
+					Logger.Write("Option style-options doesn't specify '"
 							+ s.replaceFirst(STYLE_OPTION_PREF, "") + "' (for " + s + ")");
 				}
 			}
@@ -1075,13 +1075,13 @@
 				if (points.get(0) == points.get(points.size() - 1)) {
 					// roundabout is a loop
 					if (dirIsWrong) {
-						log.warn("Roundabout " + way.getId() + " direction is wrong - reversing it (see "
+						log.write("Roundabout " + way.getId() + " direction is wrong - reversing it (see "
 								+ centre.toOSMURL() + ")");
 						way.reverse();
 					}
 				} else if (dirIsWrong) {
 					// roundabout is a line
-					log.warn("Roundabout segment " + way.getId() + " direction looks wrong (see "
+					log.write("Roundabout segment " + way.getId() + " direction looks wrong (see "
 							+ points.get(0).toOSMURL() + ")");
 				}
 			}
@@ -1232,7 +1232,6 @@
 			line.setType(replType);
 		line.setPoints(points);
 
-		
 		if (way.tagIsLikeYes(TK_ONEWAY))
 			line.setDirection(true);
 
@@ -1339,7 +1338,7 @@
 	private void addRoad(ConvertedWay cw) {
 		Way way = cw.getWay();
 		if (way.getPoints().size() < 2){
-			log.warn("road has < 2 points",way.getId(),"(discarding)");
+			log.warn("Discarding road",way.getId(),"with < 2 points");
 			return;
 		}
 
@@ -1663,7 +1662,7 @@
 		List<Integer> nodeIndices = new ArrayList<>();
 		List<Coord> points = way.getPoints();
 		if (points.size() < 2){
-			log.warn("road has < 2 points",way.getId(),"(discarding)");
+			log.warn("Discarding road",way.getId(),"with < 2 points");
 			return;
 		}
 		Way trailingWay = null;
@@ -2231,7 +2230,7 @@
 				}
 
 				if (isDeadEnd && (isDeadEndOfMultipleWays || reportDeadEnds > 1)) {
-					log.warn("Oneway road " + way.getId() + " with tags " + way.toTagString()
+					log.write("Oneway road " + way.getId() + " with tags " + way.toTagString()
 							+ ((pos == 0) ? " comes from" : " goes to") + " nowhere at " + p.toOSMURL());
 				}
 			}
Index: src/uk/me/parabola/mkgmap/osmstyle/StyleImpl.java
===================================================================
--- src/uk/me/parabola/mkgmap/osmstyle/StyleImpl.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/osmstyle/StyleImpl.java	(working copy)
@@ -239,19 +239,18 @@
 		try (InputStream is = this.getClass().getResourceAsStream("/styles/builtin-tag-list");) {
 			if (is != null) {
 				BufferedReader br = new BufferedReader(new InputStreamReader(is, StandardCharsets.UTF_8));
-				// System.out.println("Got built in list");
 				String line;
 				while ((line = br.readLine()) != null) {
 					line = line.trim();
 					if (line.startsWith("#"))
 						continue;
-					// System.out.println("adding " + line);
+
 					set.add(line);
 				}
 			}
 		} catch (IOException e) {
 			// the file doesn't exist, this is ok but unlikely
-			System.err.println("warning: built in tag list not found");
+			Logger.Write("Warning: built in tag list not found");
 		}
 		return set;
 	}
@@ -262,7 +261,7 @@
 			l = LevelInfo.DEFAULT_LEVELS;
 		LevelInfo[] levels = LevelInfo.createFromString(l);
 		if (performChecks && levels[0].getBits() <= 10) {
-			System.err.println("Warning: Resolution values <= 10 may confuse MapSource: " + l);
+			Logger.Write("Warning: Resolution values <= 10 may confuse MapSource: " + l);
 		}
 		l = generalOptions.get("overview-levels");
 		if (l != null){
@@ -270,10 +269,10 @@
 			// TODO: make sure that the combination of the two level strings makes sense
 			if (performChecks){
 				if (ovLevels[0].getBits() <= 10){
-					System.err.println("Warning: Resolution values <= 10 may confuse MapSource: " + l);
+					Logger.Write("Warning: Resolution values <= 10 may confuse MapSource: " + l);
 				}
 				if (levels[0].getLevel() >= ovLevels[ovLevels.length-1].getLevel()){
-					System.err.println("Warning: Overview level not higher than highest normal level. " + l);
+					Logger.Write("Warning: Overview level not higher than highest normal level. " + l);
 				}
 			}
 			List<LevelInfo> tmp = new ArrayList<>();
@@ -336,7 +335,7 @@
 				String val = opt.getValue();
 				if ("name-tag-list".equals(key)) {
 					if (!"name".equals(val)) {
-						System.err.println("Warning: option name-tag-list used in the style options is ignored. "
+						Logger.Write("Warning: option name-tag-list used in the style options is ignored. "
 								+ "Please use only the command line option to specify this value.");
 					}
 				} else if (OPTION_LIST.contains(key)) {
@@ -410,7 +409,7 @@
 		try {
 			baseStyles.add(new StyleImpl(location, name, props, performChecks));
 		} catch (SyntaxException e) {
-			System.err.println("Error in style: " + e.getMessage());
+			log.error("Error in style: " + e.getMessage());
 		} catch (FileNotFoundException e) {
 			// not found, try on the classpath.  This is the common
 			// case where you have an external style, but want to
@@ -420,7 +419,7 @@
 			try {
 				baseStyles.add(new StyleImpl(null, name, props, performChecks));
 			} catch (SyntaxException se) {
-				System.err.println("Error in style: " + se.getMessage());
+				log.error("Error in style: " + se.getMessage());
 			} catch (FileNotFoundException e1) {
 				log.error("Could not find base style", e);
 			}
@@ -478,7 +477,7 @@
 		}
 
 		if (version > VERSION) {
-			System.err.println("Warning: unrecognised style version " + version +
+			Logger.Write("Warning: unrecognised style version " + version +
 			", but only versions up to " + VERSION + " are understood");
 		}
 	}
@@ -546,7 +545,7 @@
 		try {
 			style = new StyleImpl(loc, name, props, WITHOUT_CHECKS);
 		} catch (SyntaxException e) {
-			System.err.println("Error in style: " + e.getMessage());
+			log.error("Error in style: " + e.getMessage());
 			throw new ExitException("Could not open style " + (name == null? "":name));
 		} catch (FileNotFoundException e) {
 			String msg = "Could not open style ";
Index: src/uk/me/parabola/mkgmap/reader/hgt/HGTList.java
===================================================================
--- src/uk/me/parabola/mkgmap/reader/hgt/HGTList.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/reader/hgt/HGTList.java	(working copy)
@@ -38,7 +38,7 @@
 		try {
 			knownHgt = loadConfig();
 		} catch (IOException e) {
-			e.printStackTrace();
+			Logger.WriteStackTrace(e);
 		}
 	} 
 	
@@ -114,7 +114,7 @@
 					
 					bs.set(getBitSetPos(lat, lon));
 				} catch (NumberFormatException e) {
-					e.printStackTrace();
+					Logger.WriteStackTrace(e);
 				}
 			}
 			return bs;
Index: src/uk/me/parabola/mkgmap/reader/hgt/HGTReader.java
===================================================================
--- src/uk/me/parabola/mkgmap/reader/hgt/HGTReader.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/reader/hgt/HGTReader.java	(working copy)
@@ -130,7 +130,7 @@
 				HGTList hgtList = HGTList.get();
 				if (hgtList != null) {
 					if (hgtList.shouldExist(lat, lon))
-						System.err.println(this.getClass().getSimpleName() + ": file " + fileName + " not found but it should exist. Height values will be 0.");
+						Logger.Write(this.getClass().getSimpleName() + ": file " + fileName + " not found but it should exist. Height values will be 0.");
 				} else { 
 					log.warn("file " + fileName + " not found. Is expected to cover sea.");
 				}
Index: src/uk/me/parabola/mkgmap/reader/osm/CoastlineFileLoader.java
===================================================================
--- src/uk/me/parabola/mkgmap/reader/osm/CoastlineFileLoader.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/reader/osm/CoastlineFileLoader.java	(working copy)
@@ -107,7 +107,7 @@
 				log.error("Coastline file " + coastlineFile + " not found.");
 			} catch (Exception exp) {
 				log.error(exp);
-				exp.printStackTrace();
+				Logger.WriteStackTrace(exp);
 			}
 		}
 		coastlinesLoaded.set(true);
Index: src/uk/me/parabola/mkgmap/reader/osm/o5m/O5mBinHandler.java
===================================================================
--- src/uk/me/parabola/mkgmap/reader/osm/o5m/O5mBinHandler.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/reader/osm/o5m/O5mBinHandler.java	(working copy)
@@ -19,6 +19,7 @@
 import java.nio.charset.StandardCharsets;
 
 import uk.me.parabola.imgfmt.app.Coord;
+import uk.me.parabola.log.Logger;
 import uk.me.parabola.mkgmap.reader.osm.Element;
 import uk.me.parabola.mkgmap.reader.osm.GeneralRelation;
 import uk.me.parabola.mkgmap.reader.osm.Node;
@@ -111,8 +112,7 @@
 				throw new IOException("wrong header byte " + start);
 			readFile();
 		} catch (IOException e) {
-			System.err.println("exception after " + countBytes + " bytes");
-			e.printStackTrace();
+			Logger.WriteStackTrace("exception after " + countBytes + " bytes", e);
 		}
 	}
 	
Index: src/uk/me/parabola/mkgmap/reader/osm/OsmMapDataSource.java
===================================================================
--- src/uk/me/parabola/mkgmap/reader/osm/OsmMapDataSource.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/reader/osm/OsmMapDataSource.java	(working copy)
@@ -167,7 +167,7 @@
 					break;
 				} catch (InstantiationException | IllegalAccessException e) {
 					// TODO Auto-generated catch block
-					e.printStackTrace();
+					Logger.WriteStackTrace(e);
 				}
 			}
 		}
Index: src/uk/me/parabola/mkgmap/reader/osm/SeaGenerator.java
===================================================================
--- src/uk/me/parabola/mkgmap/reader/osm/SeaGenerator.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/reader/osm/SeaGenerator.java	(working copy)
@@ -690,8 +690,7 @@
 		} catch (FileNotFoundException exp) {
 			log.error("Preompiled sea tile " + tileName + " not found.");
 		} catch (Exception exp) {
-			log.error(exp);
-			exp.printStackTrace();
+			Logger.WriteStackTrace(exp);
 		}
 	}
 
Index: src/uk/me/parabola/mkgmap/reader/osm/xml/OsmXmlHandler.java
===================================================================
--- src/uk/me/parabola/mkgmap/reader/osm/xml/OsmXmlHandler.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/reader/osm/xml/OsmXmlHandler.java	(working copy)
@@ -221,7 +221,7 @@
 		 */
 		@Override
 		public void fatalError(SAXParseException e) throws SAXException {
-			System.err.println("Error at line " + e.getLineNumber() + ", col "
+			Logger.Write("Error at line " + e.getLineNumber() + ", col "
 					+ e.getColumnNumber());
 			super.fatalError(e);
 		}
Index: src/uk/me/parabola/mkgmap/reader/polish/RestrictionHelper.java
===================================================================
--- src/uk/me/parabola/mkgmap/reader/polish/RestrictionHelper.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/reader/polish/RestrictionHelper.java	(working copy)
@@ -14,6 +14,7 @@
 
 import uk.me.parabola.imgfmt.app.CoordNode;
 import uk.me.parabola.imgfmt.app.net.GeneralRouteRestriction;
+import uk.me.parabola.log.Logger;
 import uk.me.parabola.mkgmap.general.MapDetails;
 
 import java.util.ArrayList;
@@ -56,7 +57,7 @@
     	if (restriction.isValid())
     		allRestrictions.add(restriction);
     	else {
-    		System.err.println(restriction);
+    		Logger.Write(restriction);
     	}
     }
 }
Index: src/uk/me/parabola/mkgmap/sea/optional/PrecompSeaGenerator.java
===================================================================
--- src/uk/me/parabola/mkgmap/sea/optional/PrecompSeaGenerator.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/sea/optional/PrecompSeaGenerator.java	(working copy)
@@ -31,6 +31,7 @@
 import java.util.concurrent.Executors;
 
 import uk.me.parabola.imgfmt.app.Coord;
+import uk.me.parabola.log.Logger;
 import uk.me.parabola.mkgmap.reader.osm.FakeIdGenerator;
 import uk.me.parabola.mkgmap.reader.osm.SeaGenerator;
 import uk.me.parabola.mkgmap.reader.osm.Way;
@@ -353,7 +354,7 @@
 								try {
 									mThread.getQueue().put(polyAsArea);
 								} catch (InterruptedException exp) {
-									exp.printStackTrace();
+									Logger.WriteStackTrace(exp);
 								}
 							}
 						}
Index: src/uk/me/parabola/mkgmap/sea/optional/PrecompSeaMerger.java
===================================================================
--- src/uk/me/parabola/mkgmap/sea/optional/PrecompSeaMerger.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/sea/optional/PrecompSeaMerger.java	(working copy)
@@ -30,6 +30,7 @@
 import java.util.concurrent.atomic.AtomicBoolean;
 
 import uk.me.parabola.imgfmt.app.Coord;
+import uk.me.parabola.log.Logger;
 import uk.me.parabola.mkgmap.reader.osm.FakeIdGenerator;
 import uk.me.parabola.mkgmap.reader.osm.GeneralRelation;
 import uk.me.parabola.mkgmap.reader.osm.MultiPolygonRelation;
@@ -112,7 +113,7 @@
 		try {
 			merge = mergeData.toMerge.poll(5, TimeUnit.MILLISECONDS);
 		} catch (InterruptedException exp) {
-			exp.printStackTrace();
+			Logger.WriteStackTrace(exp);
 		}
 		int merges = 0;
 		while (merge != null) {
@@ -218,7 +219,7 @@
 			// forward the ways to the queue of the saver thread
 			saveQueue.put(new SimpleEntry<>(mergeData.getKey(), ways));
 		} catch (InterruptedException exp) {
-			exp.printStackTrace();
+			Logger.WriteStackTrace(exp);
 		}
 
 		// signal that this tile is finished
Index: src/uk/me/parabola/mkgmap/sea/optional/PrecompSeaSaver.java
===================================================================
--- src/uk/me/parabola/mkgmap/sea/optional/PrecompSeaSaver.java	(revision 4615)
+++ src/uk/me/parabola/mkgmap/sea/optional/PrecompSeaSaver.java	(working copy)
@@ -36,6 +36,7 @@
 import uk.me.parabola.imgfmt.ExitException;
 import uk.me.parabola.imgfmt.Utils;
 import uk.me.parabola.imgfmt.app.Coord;
+import uk.me.parabola.log.Logger;
 import uk.me.parabola.mkgmap.reader.osm.SeaGenerator;
 import uk.me.parabola.mkgmap.reader.osm.Way;
 import uk.me.parabola.splitter.BinaryMapWriter;
@@ -94,7 +95,7 @@
 			try {
 				tileData = saveQueue.poll(1, TimeUnit.MINUTES);
 			} catch (InterruptedException exp) {
-				exp.printStackTrace();
+				Logger.WriteStackTrace(exp);
 			}
 			if (tileData != null) {
 				int fakeMapid = ++nextId;
